-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local logTag = "bus"
M.type = "auxilliary"
M.relevantDevice = nil
M.defaultOrder = 1000

M.doorsOpen = false
M.isKneeling = false
M.isSupensionRaised = false

local doorController = nil
local airbagController = nil

local hasSafetyInterlock = false
local passengerload = false
local lefthanddrive = false
local fueltype = false
local eaxle = false
local wheelspeed = 0
local doorLeverF1 = 0
local doorLeverF2 = 0
local doorLeverR1 = 0
local doorLeverR2 = 0
local enginedooropen = 0
local rampopenF = 0
local rampopenR = 0
local doorLeverSmoother = nil

local hasRegisteredQuickAccess = false

local htmlTexture = require("htmlTexture")
local destScreenName = nil
local destScreenCount = 1
local destScreenTimer = 0
local eventDataNew = nil
local eventDataOld = nil
local destHtmlPath = nil
local displaytextarray = {}
local displayimgarray = {}
local NSdisplayScreenName = nil
local NSHtmlPath = nil

local currentLine = {}
local timer = 0
local curWaypoint = 1
local lastApproach = -1

--local sfxprofilecounter = 0
local doorOpenSound = "file:>vehicles>capsule>sounds>capsule_dooropen.ogg"
local doorCloseSound = "file:>vehicles>capsule>sounds>capsule_doorclose.ogg"
local doornode1
local doornode2
local doornode3
local doornode4

--retarder stuff
local powerStage1 = nil
local powerStage2 = nil
local powerStage3 = nil
local powerStage4 = nil
local powerStage5 = nil
local state = 0
local retarderDevice
local gearbox = powertrain.getDevice("gearbox")
local origFriction = nil
local origRatio = nil
--

local function getDoorOpen()
  return (doorLeverF1 > 0 or doorLeverF2 > 0 or doorLeverR1 > 0 or doorLeverR2 > 0)
end

local function toggleDoorF1()
  if wheelspeed > 2 and hasSafetyInterlock then
    return
  end
  if doornode1 == nil then
    return
  end
	electrics.values.doorF1 = 1-(electrics.values.doorF1 or 0)
  if doorLeverF1 > 0 then
    doorLeverF1 = 0
	obj:playSFXOnce(doorCloseSound, doornode1, 0.3,  1)
  else
    doorLeverF1 = 1
	obj:playSFXOnce(doorOpenSound, doornode1, 0.3,  1)
  end
  electrics.values.doorLeverF1 = doorLeverF1
end

local function toggleDoorF2()
  if wheelspeed > 2 and hasSafetyInterlock then
    return
  end
  if doornode2 == nil then
    return
  end
  if rampopenF > 0 then
    guihooks.message("Please fold the wheelchair ramps first")
    return
  end
	electrics.values.doorF2 = 1-(electrics.values.doorF2 or 0)
  if doorLeverF2 > 0 then
    doorLeverF2 = 0
	obj:playSFXOnce(doorCloseSound, doornode2, 0.3,  1)
  else
    doorLeverF2 = 1
	obj:playSFXOnce(doorOpenSound, doornode2, 0.3,  1)
  end
  electrics.values.doorLeverF2 = doorLeverF2
end

local function toggleDoorR1()
  if wheelspeed > 2 and hasSafetyInterlock then
    return
  end
  if doornode3 == nil then
    return
  end
  if rampopenR > 0 then
    guihooks.message("Please fold the wheelchair ramps first")
    return
  end
	electrics.values.doorR1 = 1-(electrics.values.doorR1 or 0)
  if doorLeverR1 > 0 then
    doorLeverR1 = 0
	obj:playSFXOnce(doorCloseSound, doornode3, 0.3,  1)
  else
    doorLeverR1 = 1
	obj:playSFXOnce(doorOpenSound, doornode3, 0.3,  1)
  end
  electrics.values.doorLeverR1 = doorLeverR1
end

local function toggleDoorR2()
  if wheelspeed > 2 and hasSafetyInterlock then
    return
  end
  if doornode4 == nil then
    return
  end
	electrics.values.doorR2 = 1-(electrics.values.doorR2 or 0)
  if doorLeverR2 > 0 then
    doorLeverR2 = 0
	obj:playSFXOnce(doorCloseSound, doornode4, 0.3,  1)
  else
    doorLeverR2 = 1
	obj:playSFXOnce(doorOpenSound, doornode4, 0.3,  1)
  end
  electrics.values.doorLeverR2 = doorLeverR2
end

local function toggleallDoors()
	if getDoorOpen() then
		if doorLeverF1 > 0 then toggleDoorF1() end
		if doorLeverF2 > 0 then toggleDoorF2() end
		if doorLeverR1 > 0 then toggleDoorR1() end
		if doorLeverR2 > 0 then toggleDoorR2() end
	else
		toggleDoorF1()
		toggleDoorF2()
		toggleDoorR1()
		toggleDoorR2()
	end
end

local function toggleEngineDoor()
  if wheelspeed > 2 and hasSafetyInterlock then
    return
  end
	electrics.values.enginedoor = 1-(electrics.values.enginedoor or 0)
  if enginedooropen > 0 then
    enginedooropen = 0
  else
    enginedooropen = 1
  end
end

local function kneel()
  if wheelspeed >= 2 and hasSafetyInterlock then
    return
  end

	if lefthanddrive then
  controller.getControllerSafe('airbags').setBeamPressureLevel({'rightAxle'}, 'kneelPressure')
  else
  controller.getControllerSafe('airbags').setBeamPressureLevel({'leftAxle'}, 'kneelPressure')
  end
end

local function toggleKneel()
  if M.isKneeling then
    controller.getControllerSafe('airbags').setBeamDefault({'rightAxle', 'leftAxle'})
  else
    kneel()
  end
end

local function togglewheelchairrampF()
  if doorLeverF2 == 0 then
    guihooks.message("Please open the doors first")
    return
  end
  if doorLeverF2 > 0 and passengerload == true then
    guihooks.message("Cannot unfold the the wheelchair ramps with full passenger load")
    return
  end
  if doorLeverF2 > 0 and passengerload == false then
	electrics.values.wheelchairrampF = 1-(electrics.values.wheelchairrampF or 0)
	  if rampopenF > 0 then
		rampopenF = 0
	  else
		rampopenF = 1
	  end
  end
end

local function togglewheelchairrampR()
  if doorLeverR1 == 0 then
    guihooks.message("Please open the doors first")
    return
  end
  if doorLeverR1 > 0 and passengerload == true then
    guihooks.message("Cannot unfold the the wheelchair ramps with full passenger load")
    return
  end
  if doorLeverR1 > 0 and passengerload == false then
	electrics.values.wheelchairrampR = 1-(electrics.values.wheelchairrampR or 0)
	  if rampopenR > 0 then
		rampopenR = 0
	  else
		rampopenR = 1
	  end
  end
end

local function geCallback(event,data)
  data.event = event
  --transform userdata into table
  data.pos = vec3(obj:getPosition()):toTable()
  data.rot = quat(obj:getRotation()):toTable()
  obj:queueGameEngineLua("if core_busRouteManager then core_busRouteManager.onBusUpdate("..dumps(data)..") end")
end

local function updateGFX(dt)
	if controller.getController('hybridPowertrain') == nil and fueltype == 1 then
		guihooks.message("Arcanox Core is required for hybrid variant. Please download it from the repo.", 600, nil, "error")
	end
	
  --retarder stuff
  if retarderDevice then
    if electrics.values.throttle > 0.1 then
      state = 0
    elseif electrics.values.wheelspeed <= 0.5 then
        state = 0
    elseif fueltype == 0 and gearbox then
      if gearbox.gearRatios[gearbox.gearIndex] == 0 then
        state = 0
        return false
      end
    end
	if fueltype == 1 then
		if state > 0 and electrics.values.wheelspeed > 22.2 then
			retarderDevice.counterRingGearRatio = origRatio / 2
		else
			retarderDevice.counterRingGearRatio = origRatio
		end
	end
      if state == 0 then
          retarderDevice.friction = origFriction
          --guihooks.message("state 0", 10, "vehicle.retarder.state")
      elseif state == 1 then
          retarderDevice.friction = origFriction + powerStage1
          --guihooks.message("state 1", 10, "vehicle.retarder.state")
      elseif state == 2 then
          retarderDevice.friction = origFriction + powerStage2
          --guihooks.message("state 2", 10, "vehicle.retarder.state")
      elseif state == 3 then
          retarderDevice.friction = origFriction + powerStage3
          --guihooks.message("state 3", 10, "vehicle.retarder.state")
      elseif state == 4 then
          retarderDevice.friction = origFriction + powerStage4
          --guihooks.message("state 4", 10, "vehicle.retarder.state")
      elseif state == 5 then
          retarderDevice.friction = origFriction + powerStage5
          --guihooks.message("state 5", 10, "vehicle.retarder.state")
      end

    --input part
    if electrics.values.retarder_up == 1 then
        state = state + 1
        electrics.values.retarder_up = 0
    elseif electrics.values.retarder_down == 1 then
        state = state - 1
        electrics.values.retarder_down = 0
    elseif electrics.values.retarder_max == 1 then
        state = 5
        electrics.values.retarder_max = 0
    elseif electrics.values.retarder_off == 1 then
        state = 0
        electrics.values.retarder_off = 0
    end

    -- avoid state jumping to anything over 5 or under 0
    if state >= 6 then
        state = 5
    elseif state <= -1 then
        state = 0
    end

    if state >= 3 then
      electrics.values.brakelights = 1
    end
	electrics.values.retarder = state
  end
  --
  
  --if doorControllerF1 or doorControllerF2 or doorControllerR1 or doorControllerR2 then
    local frontDoorOpen = (doorLeverF1 > 0 or doorLeverF2 > 0)
    local rearDoorOpen = (doorLeverR1 > 0 or doorLeverR2 > 0)
    M.doorsOpen = frontDoorOpen or rearDoorOpen
  --end
  --log("I",logTag,tostring(M.doorsOpen)..tostring(frontDoorOpen)..tostring(rearDoorOpen))

  if airbagController then
	if lefthanddrive then
    M.isKneeling = airbagController.isBeamGroupAtPressureLevel("rightAxle","kneelPressure")
    M.isSupensionRaised = airbagController.isBeamGroupAtPressureLevel("rightAxle","maxPressure")
	else
    M.isKneeling = airbagController.isBeamGroupAtPressureLevel("leftAxle","kneelPressure")
    M.isSupensionRaised = airbagController.isBeamGroupAtPressureLevel("leftAxle","maxPressure")
	end
  end

  wheelspeed = electrics.values.wheelspeed or 0

  if (getDoorOpen() or M.isKneeling or enginedooropen > 0) and wheelspeed < 2 and hasSafetyInterlock then
    electrics.values.throttle = 0
    electrics.values.brake = 1
  end
  
  if fueltype == 2 and powertrain.getDevice("rearMotor").isDisabled then
	electrics.values.throttle = 0
	if electrics.values["running"] then
		electrics.toggleIgnitionLevelOnUp()
	end
  end
  
  if fueltype ~= 2 and eaxle then
    electrics.values.throttle = 0
    electrics.values.brake = 1
  end

  electrics.values.dooropen = M.doorsOpen and 1 or 0
  electrics.values.kneel = M.isKneeling and 1 or 0
  electrics.values.rideheight = M.isSupensionRaised and 1 or 0
  --electrics.values.doorLeverF1 = doorLeverF1
  --electrics.values.doorLeverF2 = doorLeverF2
  --electrics.values.doorLeverR1 = doorLeverR1
  --electrics.values.doorLeverR2 = doorLeverR2

	--multi-phase display
	if #(displaytextarray) > 1 then
		destScreenTimer = destScreenTimer + dt
		if destScreenName and destScreenTimer >= tonumber(eventDataOld.interval) then
		  eventDataNew.displaytext = displaytextarray[destScreenCount]
		  eventDataNew.displayimg = displayimgarray[destScreenCount]
		  destScreenCount = destScreenCount + 1
		  if destScreenCount > #(displaytextarray) then destScreenCount = 1 end
		  htmlTexture.call(destScreenName, "update", eventDataNew)
		  destScreenTimer = destScreenTimer % tonumber(eventDataOld.interval)
			--dump(destScreenCount)
			--dump(#(displaytextarray))
			--dump(displaytextarray[1])
			--dump(displaytextarray[2])
			--dump(displaytextarray[3])
		end
    end
	
  timer = timer + dt
  if timer > 0.5 then
    timer = timer % 0.5
    if currentLine.tasklist and currentLine.tasklist[curWaypoint] then
      --log("E",logTag,"pos ="..dumps(currentLine.tasklist[cur][3]))
      local dist = vec3(currentLine.tasklist[curWaypoint][3] ):distance(vec3(obj:getPosition()))
      -- log("E",logTag,"dist ="..tostring(dist))
      if dist < 75 and curWaypoint ~= lastApproach then
        controller.onGameplayEvent("bus_onApproachStop", {triggerName=currentLine.tasklist[curWaypoint][1]})
        lastApproach = curWaypoint
      end
    end
  end
end

local function reset()
	if controller.getController('hybridPowertrain') == nil and fueltype == 1 then guihooks.message("Arcanox Core is required for hybrid variant. Please download it from the repo.", 600, nil, "error") end
  M.doorsOpen = false
  M.isKneeling = false
  M.isSupensionRaised = false
  doorLeverF1 = 0
  doorLeverF2 = 0
  doorLeverR1 = 0
  doorLeverR2 = 0
  enginedooropen = 0
  rampopenF = 0
  rampopenR = 0
  electrics.values.dooropen = 0
  electrics.values.doorLeverF1 = 0
  electrics.values.doorLeverF2 = 0
  electrics.values.doorLeverR1 = 0
  electrics.values.doorLeverR2 = 0
  electrics.values.doorF1 = 0
  electrics.values.doorF2 = 0
  electrics.values.doorR1 = 0
  electrics.values.doorR2 = 0
  electrics.values.enginedoor = 0
  electrics.values.kneel = 0
  electrics.values.rideheight = 0
  electrics.values.wheelchairrampF = 0
  electrics.values.wheelchairrampR = 0
  electrics.values.stoprequest = 0

  timer = 0
  curWaypoint = 1
  lastApproach = -1
  if lefthanddrive then
	  for k, node in pairs (v.data.nodes) do
		if node.name == "st3rra" then doornode1=k  end
		if node.name == "st5rra" then doornode2=k  end
		if node.name == "st9rra" then doornode3=k  end
		if node.name == "st15rr3a" then doornode4=k  end
	  end
  else
	  for k, node in pairs (v.data.nodes) do
		if node.name == "st3lla" then doornode1=k  end
		if node.name == "st5lla" then doornode2=k  end
		if node.name == "st9lla" then doornode3=k  end
		if node.name == "st15ll3a" then doornode4=k  end
	  end
  end
end

local function registerQuickAccess()
  if not hasRegisteredQuickAccess and core_quickAccess ~= nil then
    core_quickAccess.addEntry({ level = '/', title = 'Bus', icon="material_directions_bus", ["goto"] = '/bus/'} )
    core_quickAccess.addEntry({ level = '/bus/', generator = function(entries)
          local e = { title = 'ui.radialmenu2.bus.kneel', icon="material_arrow_downward", onSelect = function() toggleKneel() return {'reload'} end }
          if electrics.values.kneel == 1 then e.color = '#ff6600' end
          table.insert(entries, e)

          e = { title = 'ui.radialmenu2.bus.doors', icon="radial_hinges", onSelect = function() toggleallDoors() return {'reload'} end }
          if getDoorOpen() then e.color = '#ff6600' end
          table.insert(entries, e)
        end})

    hasRegisteredQuickAccess = true
  end
end

local function init(jbeamData)
  fueltype = jbeamData.fueltype or 0
  --retarder stuff
  if fueltype ~= 2 then
    retarderDevice = powertrain.getDevice("gearbox")
  else
    retarderDevice = powertrain.getDevice("rearMotor")
  end
  if retarderDevice then
    powerStage1 = jbeamData.powerStage1 or 1000
    powerStage2 = jbeamData.powerStage2 or 1500
    powerStage3 = jbeamData.powerStage3 or 2000
    powerStage4 = jbeamData.powerStage4 or 2500
    powerStage5 = jbeamData.powerStage5 or 3000
    origFriction = retarderDevice.friction
    origRatio = retarderDevice.counterRingGearRatio or 210/160
  end
  --
  
  hasSafetyInterlock = jbeamData.hasSafetyInterlock or false
  passengerload = jbeamData.passengerload or false
  lefthanddrive = jbeamData.lefthanddrive or false
  eaxle = jbeamData.eaxle or false

  M.doorsOpen = false
  M.isKneeling = false
  M.isSupensionRaised = false
  electrics.values.dooropen = 0
  electrics.values.doorLeverF1 = 0
  electrics.values.doorLeverF2 = 0
  electrics.values.doorLeverR1 = 0
  electrics.values.doorLeverR2 = 0
  electrics.values.doorF1 = 0
  electrics.values.doorF2 = 0
  electrics.values.doorR1 = 0
  electrics.values.doorR2 = 0
  electrics.values.enginedoor = 0
  electrics.values.kneel = 0
  electrics.values.rideheight = 0
  electrics.values.wheelchairrampF = 0
  electrics.values.wheelchairrampR = 0
  electrics.values.stoprequest = 0

  --doorLeverSmoother = newTemporalSmoothing(2,2)
  doorLeverF1 = 0
  doorLeverF2 = 0
  doorLeverR1 = 0
  doorLeverR2 = 0

  registerQuickAccess()

  if jbeamData.nextStopNode and v.data[jbeamData.nextStopNode] then
    local nsData = v.data[jbeamData.nextStopNode]
    NSdisplayScreenName = nsData.NSmaterialName
    NSHtmlPath = nsData.NShtmlPath
    local NSwidth = nsData.NStextureWidth or 512
    local NSheight = nsData.NStextureHeight or 384

    if NSHtmlPath and NSdisplayScreenName then
      htmlTexture.create(NSdisplayScreenName, NSHtmlPath, NSwidth, NSheight, 15, 'automatic')
    else
      log("E", logTag, "Got no html or material name path for NextStop, no HTML texture created!!!...")
    end
  end

  if jbeamData.destinationSignNode and v.data[jbeamData.destinationSignNode] then
    local destData = v.data[jbeamData.destinationSignNode]
    destScreenName = destData.destMaterialName or destScreenName
    destHtmlPath = destData.destHtmlPath
    local destWidth = destData.destTextureWidth or 256
    local destHeight = destData.destTextureHeight or 16
    local destText = destData.destText or ""
    local destRoute = destData.destRoute or ""
    local destExtra1 = destData.destExtra1 or ""
    local destExtra2 = destData.destExtra2 or ""
    local destInt = destData.destInt or 3
    local destImg = destData.destImg or 0
    local destExtra1Img = destData.destExtra1Img or 0
    local destExtra2Img = destData.destExtra2Img or 0
    local routeImg = destData.routeImg or 0

    if destHtmlPath and destScreenName then
      htmlTexture.create(destScreenName, destHtmlPath, destWidth, destHeight, 4, 'automatic')
		eventDataNew = {direction = destText, routeID = destRoute, displaytext = destText, extratext1 = destExtra1, extratext2 = destExtra2, interval = destInt, displayimg = destImg, destimg = destImg, extratext1img = destExtra1Img, extratext2img = destExtra2Img, routeimg = routeImg}
		eventDataOld = eventDataNew
      htmlTexture.call(destScreenName, "update", eventDataOld )
    else
      log("E", logTag, "Got no html path or material name for destination, no HTML texture created!!!...")
    end
  end
  if lefthanddrive then
	  for k, node in pairs (v.data.nodes) do
		if node.name == "st3rra" then doornode1=k  end
		if node.name == "st5rra" then doornode2=k  end
		if node.name == "st9rra" then doornode3=k  end
		if node.name == "st15rr3a" then doornode4=k  end
	  end
  else
	  for k, node in pairs (v.data.nodes) do
		if node.name == "st3lla" then doornode1=k  end
		if node.name == "st5lla" then doornode2=k  end
		if node.name == "st9lla" then doornode3=k  end
		if node.name == "st15ll3a" then doornode4=k  end
	  end
	end
end

local function initSecondStage()
  doorControllerF1 = controller.getController("doorF1")
  doorControllerF2 = controller.getController("doorF2")
  doorControllerR1 = controller.getController("doorR1")
  doorControllerR2 = controller.getController("doorR2")
  airbagController = controller.getController("airbags")
end

local function recalculateStopList()
  local sl = {}
  for i = curWaypoint, #currentLine.tasklist, 1 do
    table.insert(sl, currentLine.tasklist[i][2])
  end
  return sl
end

--duplicate code at scenario/busdriver.lua:62
local function isTriggerOnBusLine(tasks,tname)
  for _,v in pairs(tasks) do
    if v[1] == tname then return true end
  end
  return false
end

local function uiSetLine(routeID, variance)
  obj:queueGameEngineLua("if core_busRouteManager then core_busRouteManager.setLine("..tostring(obj:getID())..","..dumps(routeID)..","..dumps(variance)..") end")
end

local function onGameplayEvent(eventName, eventData)
  if eventName == "bus_onRouteChange" then
    if destScreenName then
		eventDataNew = eventData
		eventDataOld = eventData
		destScreenCount = 1
		destScreenTimer = tonumber(eventDataOld.interval)
		displaytextarray = {}
		displayimgarray = {}
		  table.insert(displaytextarray, eventDataOld.direction)
		  if eventDataOld.extratext1 ~= "" then table.insert(displaytextarray, eventDataOld.extratext1) end
		  if eventDataOld.extratext2 ~= "" then table.insert(displaytextarray, eventDataOld.extratext2) end
		  table.insert(displayimgarray, eventDataOld.destimg)
		  if eventDataOld.extratext1 ~= "" then table.insert(displayimgarray, eventDataOld.extratext1img) end
		  if eventDataOld.extratext2 ~= "" then table.insert(displayimgarray, eventDataOld.extratext2img) end
		  eventDataNew.displaytext = displaytextarray[1]
		  eventDataNew.displayimg = displayimgarray[1]
		htmlTexture.call(destScreenName, "update", eventDataNew)
    end
    if NSdisplayScreenName then
      htmlTexture.call(NSdisplayScreenName, "updateDisplay", eventData)
    end
    guihooks.trigger('BusDisplayUpdate', eventData)

  elseif eventName == "bus_onDepartedStop" then
    if not currentLine.tasklist or #currentLine.tasklist == 0 then
      log("E",logTag,"No tasklist!")
      return
    end
    if not isTriggerOnBusLine(currentLine.tasklist, eventData.triggerName) then return end

    --fix spawning the bus in last busstop trigger, you still cannot go through, only exit once
    if currentLine.tasklist[#currentLine.tasklist][1] == eventData.triggerName and curWaypoint<#currentLine.tasklist  then
      return
    end

    for i=1, #currentLine.tasklist, 1 do
      if currentLine.tasklist[i][1] == eventData.triggerName then curWaypoint=i;break end
    end
    curWaypoint = curWaypoint + 1

    local stopList = recalculateStopList()

    if NSdisplayScreenName then
      htmlTexture.call(NSdisplayScreenName, "updateDisplay", stopList)
    end
    guihooks.trigger('BusDisplayUpdate', stopList)

    geCallback("onDepartedStop",eventData)
    electrics.values.stoprequest = 0

  elseif eventName == "bus_onAtStop" then
    if not currentLine.tasklist or #currentLine.tasklist == 0 then
      log("E",logTag,"No tasklist!")
      return
    end
    if not isTriggerOnBusLine(currentLine.tasklist, eventData.triggerName) then return end

    for i = 1, #currentLine.tasklist, 1 do
      if currentLine.tasklist[i][1] == eventData.triggerName then curWaypoint = i; break end
    end

    geCallback("onAtStop", eventData)

  elseif eventName == "bus_onApproachStop" then
    -- guihooks.trigger('Message', {ttl = 5, msg = 'onApproachStop', icon = 'directions_bus'})

    for i = 1, #currentLine.tasklist, 1 do
      if currentLine.tasklist[i][1] == eventData.triggerName then curWaypoint = i; break end
    end

    geCallback("onApproachStop", eventData)

  elseif eventName == "bus_setLineInfo" then
    -- log("I",logTag..".setLineInfo","eventData received in lua veh ="..dumps(eventData))
    reset()
    currentLine = eventData
    controller.onGameplayEvent("bus_onRouteChange", currentLine)
    -- if currentLine.tasklist[1] then
    --   controller.onGameplayEvent("bus_onAtStop",{triggerName=currentLine.tasklist[1][1]})
    -- end

  elseif eventName == "bus_onTriggerTick" then
    eventData.speed = electrics.values.wheelspeed * 3.6
    --eventData.speed = electrics.values.wheelspeed * 2
    eventData.bus_dooropen = M.doorsOpen
    --eventData.bus_dooropen = getDoorOpen()
    eventData.bus_kneel = M.isKneeling
    geCallback("onTriggerTick",eventData)
  elseif eventName == "bus_onSetStopRequest" then
    if eventData then 
      guihooks.trigger('SetStopRequest', eventData)
	  electrics.values.stoprequest = 1
    end
  end
end

M.init = init
M.initSecondStage = initSecondStage
M.reset = reset
M.updateGFX = updateGFX
M.onGameplayEvent = onGameplayEvent

M.toggleallDoors = toggleallDoors
M.toggleDoorF1 = toggleDoorF1
M.toggleDoorF2 = toggleDoorF2
M.toggleDoorR1 = toggleDoorR1
M.toggleDoorR2 = toggleDoorR2
M.toggleEngineDoor = toggleEngineDoor
M.toggleKneel = toggleKneel
M.togglewheelchairrampF = togglewheelchairrampF
M.togglewheelchairrampR = togglewheelchairrampR

M.uiSetLine = uiSetLine

return M
