-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxiliary"
M.relevantDevice = nil

local saveFilePath = "settings/capsule/gauges.json"

local htmlTexture = require("htmlTexture")

local min = math.min
local max = math.max

local gaugesScreenName = nil
local htmlPath = nil

local updateFPS = 30
local updateTimer = 0
local saveTimer = 0

local gaugeunitvalue = 0
local distanceTotalm = 0
local distanceTotalmOld = 0
local hudOn = 1
local hudsmoother = nil

local motors = {}
local currentPower = 0

local function updateGFX(dt)
    local data = {}

	electrics.values["hud"] = hudsmoother:getUncapped(hudOn, dt)
  updateTimer = updateTimer + dt
  if updateTimer > (1 / updateFPS) then
    data.dategauge = os.date("%d") .. " " .. string.upper(os.date("%b")) .. " " .. os.date("%Y") .. " " .. os.date("%H") .. ":" .. os.date("%M")
	gaugeunitvalue = electrics.values["gaugeunit"]
	data.gaugeunit = gaugeunitvalue
	data.hud = hudOn
	
	data.speedometergauge = electrics.values["wheelspeed"] * 3.6
	data.rpm = electrics.values["rpm"]
	data.enginerunning = electrics.values["engineRunning"]
	
	data.signal_l = electrics.values["signal_L"]
	data.signal_r = electrics.values["signal_R"]
	data.lowfuel = electrics.values["lowfuel"]
	data.battery = electrics.values["battery"]
	data.state = electrics.values["retarder"]
	
	data.fuel = electrics.values["fuel"]
	data.watertemp = electrics.values["watertemp"]
	
	data.kneel = electrics.values["kneel"]
	data.doorLeverF1 = electrics.values["doorLeverF1"]
	data.doorLeverF2 = electrics.values["doorLeverF2"]
	data.doorLeverR1 = electrics.values["doorLeverR1"]
	data.doorLeverR2 = electrics.values["doorLeverR2"]
	data.wheelchairrampF = electrics.values["wheelchairrampF"]
	data.wheelchairrampR = electrics.values["wheelchairrampR"]
	data.enginedoor = electrics.values["enginedoor"]
	
	distanceTotalm = distanceTotalm + electrics.values.wheelspeed * dt
	electrics.values["odometer"] = math.floor(distanceTotalm / 100) / 10
	data.odometer =  electrics.values["odometer"]
	
	data.fueltype = electrics.values["fueltype"]
	data.batteryLevel = electrics.values["batteryLevel"]
	data.kw = electrics.values["kw"]

  for _, motor in ipairs(motors) do
    currentPower = motor.outputTorque1 * motor.outputAV1
  end
  data.kwev = math.floor(currentPower / 1000)

    htmlTexture.call(gaugesScreenName, "updateData", data)
    updateTimer = 0
  end
	
	saveTimer = saveTimer + dt
	if saveTimer >= 1 and math.floor(distanceTotalm) ~= distanceTotalmOld then
		distanceTotalmOld = math.floor(distanceTotalm)
		jsonWriteFile(saveFilePath, {gaugeunit = gaugeunitvalue, odometer = math.floor(distanceTotalm), hud = hudOn}, true)
		saveTimer = 0
		--dump(gaugeunitvalue)
		--dump(math.floor(distanceTotalm))
	end
end

local function togglegaugeunits()
	electrics.values["gaugeunit"] = 1 - (electrics.values["gaugeunit"] or 0)
	gaugeunitvalue = electrics.values["gaugeunit"]
    jsonWriteFile(saveFilePath, {gaugeunit = gaugeunitvalue, odometer = math.floor(distanceTotalm), hud = hudOn}, true)
end

local function togglehud()
	hudOn = 1 - (hudOn or 0)
    jsonWriteFile(saveFilePath, {gaugeunit = gaugeunitvalue, odometer = math.floor(distanceTotalm), hud = hudOn}, true)
end

local function init(jbeamData)
	gaugesScreenName = jbeamData.materialName
	htmlPath = jbeamData.htmlPath
	local width = jbeamData.textureWidth or 1024
	local height = jbeamData.textureHeight or 1024

    htmlTexture.create(gaugesScreenName, htmlPath, width, height, updateFPS, "automatic")
	
	electrics.values["odometer"] = 0
	
	electrics.values["fueltype"] = jbeamData.fueltype or 0
	electrics.values["gaugeunit"] = 0
	electrics.values["hud"] = 1
	hudsmoother = newTemporalSmoothing(2, 2)

	motors = powertrain.getDevicesByType("electricMotor")
	
    local readdata = jsonReadFile(saveFilePath)
    if readdata then
		gaugeunitvalue = readdata.gaugeunit
        distanceTotalm = readdata.odometer
        distanceTotalmOld = readdata.odometer
		hudOn = readdata.hud
		electrics.values["odometer"] = math.floor(distanceTotalm / 100) / 10
		electrics.values["gaugeunit"] = gaugeunitvalue
		electrics.values["hud"] = hudOn
    end
end

M.init = init
M.updateGFX = updateGFX
M.togglegaugeunits = togglegaugeunits
M.togglehud = togglehud

return M
